function [t y NewIterat] = impl_tayNewton(step, tmax, lambda, ORD, TOL)
% [t y NewIterat] = impl_tayNewton(step, tmax, lambda, ord, TOL)
% Newton method: y(j+1)=y(j)-f(y(j))/f'(y(j))
% derivation f'(y(j)) computed using differential formula

hN=1e-3; % step size in differential formula
t = (0:step:tmax);
y = zeros(length(t), 1);
NewIterat = zeros(length(t), 1);
y(1) = 1; % initial condition y(0) = 1
for i = 1:(length(t)-1)
    yj=inf; %inf
    yjj=y(i);
    NewIter=0;
    while (abs(yj-yjj)>TOL && NewIter<10) % Newton iteration
        yj=yjj;
        DY=yj;
        DYN=yj+hN;
        f=-yj+y(i);
        fN=-DYN+y(i);
        for a = 1:ORD % recurrent calculation of Taylor series terms
            DY  = -(step/a)*lambda*DY;
            DYN = -(step/a)*lambda*DYN;
            f=f-DY;
            fN=fN-DYN;
        end
        derf=(fN-f)/hN; % differential formula
        yjj=yj-f/derf; % Newton method
        NewIter=NewIter+1;
    end
    y(i+1)=yjj;
    NewIterat(i+1)=NewIter;
end