function implicit
%close all;
global lambda;
lambda = -1000;  % constant in ODE: y'=lambda*y
step = 0.1; % integration step: h
TMAX = 1;  % interval of solution [0, TMAX]
figure;
fix = false; % auxilary variable for plotting
[t y] = impl_eul(step, TMAX, lambda);
fix = fix || max(abs(y)) > 10;
plot(t,y, 'y');
hold on;
[t y] = impl_trap(step, TMAX, lambda);
plot(t, y, 'b');
fix = fix || max(abs(y)) > 10;
plot(t, exp(lambda * t), 'r');
[t y] = impl_tay(step, TMAX, lambda, 2); % ORD = 2
fix = fix || max(abs(y)) > 10;
plot(t, y, 'g');
[t y] = impl_tay(step, TMAX, lambda, 3); % ORD = 3
fix = fix || max(abs(y)) > 10;
plot(t, y, 'c');
[t y] = impl_tay(step, TMAX, lambda, 4); % ORD = 4
fix = fix || max(abs(y)) > 10;
plot(t, y, 'm');
tic;
[t y NewIter2] = impl_tayNewtonSymb(step, TMAX, lambda, 4, 1e-4); % ORD = 4, TOL = 1e-4
fprintf('Elapsed time of Taylor series + Newton iterations (symbolic) ORD=4: %i seconds. \n', toc);
fix = fix || max(abs(y)) > 10;
plot(t, y, 'vm');
%tic;
[t y] = impl_tay(step, TMAX, lambda, 10); % ORD = 10
%toc
fix = fix || max(abs(y)) > 10;
plot(t, y, 'k');
tic;
[t y NewIter] = impl_tayNewton(step, TMAX, lambda, 10, 1e-4); % ORD = 10, TOL = 1e-4
fprintf('Elapsed time of Taylor series + Newton iterations (dif. formulae) ORD=10: %i seconds. \n', toc);
fix = fix || max(abs(y)) > 10;
plot(t, y, 'vk');
tic;
[t y] = ode15s(@ev, 0:step:TMAX, 1);
fprintf('Elapsed time of ''ode15s'' solver: %i seconds. \n', toc);
plot(t, y, 'g+');
legend('Euler', 'Trapezoidal', 'Exact solution', 'Taylor ORD = 2', 'Taylor ORD = 3', 'Taylor ORD = 4', 'Taylor Newton (symbolic) ORD = 4', 'Taylor ORD = 10', 'Taylor Newton ORD = 10', 'ode15s');
xlabel('t');
ylabel('y');
title(sprintf('Implicit numerical methods (lambda = %i, z = %i)', lambda, step*lambda));
if fix
    axis([0 1 -10 10]);
end
hold off;
disp(['Number of Newton iterations: ', num2str(NewIter')]);
fprintf('The sum of Newton iterations: %i \n', sum(NewIter));
disp(['Number of Newton iterations (symbolic): ', num2str(NewIter2')]);
fprintf('The sum of Newton iterations (symbolic): %i \n ---------------------------------\n', sum(NewIter2));

function dydt = ev(t, y)
    global lambda;
    dydt = lambda * y;